/*
 * TuringMachinePanel.java
 *
 * Created on January 16, 2001, 4:26 PM
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 *
 * @author  Dirk Zitterell
 * @version  0.1
 */
public class TuringMachinePanel extends javax.swing.JPanel {
    
    /** Creates new TuringMachinePanel */
    public TuringMachinePanel(int w, int h) {
        Fields.width = ( w >= 0 ? w : 600 );
        Fields.height = ( h >= 0 ? h : 100);
        this.tapeWidth = Fields.width - (2 * Fields.margin); 
        Fields.numberOfVisibleFields = tapeWidth / Fields.tapeFieldWidth;
        this.middleX = Fields.width / 2;
        this.tapeX = Fields.margin;
        this.tapeY = Fields.margin;
        this.headX = (Fields.width - Fields.tapeFieldWidth) / 2;
        this.headY = tapeY;
    }
    
    
    /* Paints the tape */
    private void paintTape(Graphics g) {
        g.setColor(Color.black);
        g.drawString(((NodeField) (Fields.tape.current.getContent())).getIndex() + " Field Index", (Fields.width / 2), (Fields.margin / 2));
        g.setColor(Color.white);
        g.fillRect(tapeX, tapeY, tapeWidth, Fields.tapeFieldHeight + 1);
        g.setColor(Color.black);
        g.drawRect(tapeX - 1, tapeY - 1, tapeWidth + 1 , Fields.tapeFieldHeight + 2);
        paintTapeFields(g);
    }
    
    
    /* paints the write-read head */
    private void paintWriteReadHead(Graphics g) {
        g.setColor(Color.blue);
        g.drawRect(headX - 1, headY - 1, Fields.tapeFieldWidth + 2, Fields.tapeFieldHeight + 2);
        g.drawRect(headX, headY, Fields.tapeFieldWidth, Fields.tapeFieldHeight);
        g.drawRect(middleX, headY + Fields.tapeFieldHeight, 1, 10);
        g.drawRect(headX - 1, headY + Fields.tapeFieldHeight + 9, Fields.tapeFieldWidth + 2, Fields.tapeFieldHeight + 2);
        g.drawRect(headX, headY + Fields.tapeFieldHeight + 10, Fields.tapeFieldWidth, Fields.tapeFieldHeight);
        
    }
    
    
    /* Paints the TapeFields */
    private void paintTapeFields(Graphics g) {
        TapeNode tn1 = Fields.tape.current;
        TapeNode tn2 = Fields.tape.current.getPreviousNode(Fields.tape.reversed);
        for (int i = 0; i < (Fields.numberOfVisibleFields / 2); i++) {
            g.drawString("" + ((NodeField) tn1.getContent()).getSymbol(), headX + 5 + (i * Fields.tapeFieldWidth), headY + 15);
            tn1 = tn1.getNextNode(Fields.tape.reversed);
        }
        
        for (int i = 1; i < (Fields.numberOfVisibleFields / 2); i++) {
            g.drawString("" + ((NodeField) tn2.getContent()).getSymbol(), headX + 5 - (i * Fields.tapeFieldWidth), headY + 15);
            tn2 = tn2.getPreviousNode(Fields.tape.reversed);
        }
        
        
    }
  

    /* Update component */
    public void update(Graphics g) {
        g.setColor(Color.gray); 
	g.fillRect(0, 0, (int) this.getSize().getWidth(), (int) this.getSize().getHeight());
	paint(g);
    }

    
    /*  Repaints this panel */
    public void paint(Graphics g) {
        g.setColor(new Color(204, 204, 204)); 
	g.fillRect(0, 0, (int) this.getSize().getWidth(), (int) this.getSize().getHeight());
	paintTape(g);
	paintWriteReadHead(g);
	
    }
    
    
    /* Scrolls the tape one field to the right */
    public void moveLeft() {
        if ((((NodeField) Fields.tape.current.getContent()).getIndex() - (Fields.numberOfVisibleFields / 2)) < ((NodeField) Fields.tape.leftEnd.getContent()).getIndex()) {
            Fields.tape.insertLeftEnd(new NodeField(Fields.symbolEmptyField, ((NodeField) Fields.tape.leftEnd.getContent()).getIndex() - 1));
            System.out.println("New Field at the left end inserted.");
        }
        Fields.tape.current = Fields.tape.current.getPreviousNode(Fields.tape.reversed);
        repaint();
    }
    
    
    /* Scrolls the tape one field to the left */
    public void moveRight() {
        if ((((NodeField) Fields.tape.current.getContent()).getIndex() + (Fields.numberOfVisibleFields / 2)) > ((NodeField) Fields.tape.rightEnd.getContent()).getIndex()) {
            Fields.tape.insertRightEnd(new NodeField(Fields.symbolEmptyField, ((NodeField) Fields.tape.rightEnd.getContent()).getIndex() + 1));
            System.out.println("New Field at the right end inserted.");
        }
        Fields.tape.current = Fields.tape.current.getNextNode(Fields.tape.reversed);
        repaint();
    }

    
    /* Returns the preferred size of this component */
    public Dimension getPreferredSize() {
        return new Dimension(600, 75); 
    }
    
    
    /* Returns the minimum size of this component */
    public Dimension getMinimumSize() {
        return getPreferredSize(); 
    }
    
    // custom Variables declaration
    private int middleX;
    private  int tapeX;
    private  int tapeY;
    private  int headX;
    private  int headY;
    private  int tapeWidth;
    // End of custom Variables declaration
    
}
